<?php
/**
 * mysqli.class.php MYSQLI数据库实现类
 *
 * @author		大潇  https://daxiao.cc/
 * @php_version	php >= 8.0
 * @date		2024-04-22
 */

final class db_factroy {

	/**
	 * 数据库连接资源句柄
	 */
	private object $conn;

	/**
	 * 数据库连接地址
	 */
	private string $host = "";

	/**
	 * 数据库用户账号
	 */
	private string $user = "";

	/**
	 * 数据库用户密码
	 */
	private string $pwd = "";

	/**
	 * 数据库名称
	 */
	private string $db = "";

	/**
	 * 数据库连接端口
	 */
	private int $port = 3306;

	/**
	 * 结果集
	 */
	private int|object $result;

	/**
	 * 预置操作表名
	 */
	public string $table_name;

	/**
	 * 构造函数，建立数据库连接
	 */
	public function __construct(){
		$this->conn = @new mysqli($this->host, $this->user, $this->pwd, $this->db, $this->port);
		if(!$this->conn){
			echo "Can not connect to MySQL server<br/>";
			echo "Error Code：".mysqli_connect_errno()."<br/>";
			echo "Error Message：".mysqli_connect_error();
			exit;
		}
		$this->conn->query("SET NAMES UTF8");
	}
	
	/**
	 * 析构资源释放
	 */
	public function __destruct(){
		$this->mysql_close();
	}

	/**
	 * 查询获取结果集
	 * @param $sql			SQL语句
	 * @param $index		返回结果集格式
	 *						1 表示从结果集中取得所有行作为关联数组 key=>value
	 *						2 表示从结果集中取得所有行作为数字数组 0=>value
	 *						3 表示从结果集中取得所有行作为关联数组和数字数组(二者都有) 0=>value key=>value
	 * @return array		查询结果集数组
	 */
	public function fetch_all(object $result, int $index = 1):Array{
		$resulttype = match($index){
			1 => MYSQLI_ASSOC,
			2 => MYSQLI_NUM,
			3 => MYSQLI_BOTH,
			default => MYSQLI_ASSOC,
		};
		$info=$result->fetch_all($resulttype);
		$this->free_result($result);
		return $info;
	}

	/**
	 * 执行sql查询
	 * @param $where 		查询条件[例`name`='$name']
	 * @param $field 		需要查询的字段值[例`name`,`gender`,`birthday`]
	 * @param $order 		排序方式	[例：id desc，默认为空]
	 * @param $limit 		返回结果范围[例：10 或着 10,10，默认为空]
	 * @param $table 		数据表
	 * @return array		查询结果集数组
	 */
	public function select(string $where='', string $field='*', string $order='', string $limit='', string $table=''):Bool|Array{
		$table = $this->check_table($table);
		$where = empty($where) ? '' : ' where '.$where;
		$order = empty($order) ? '' : ' order by '.$order;
		$limit = empty($limit) ? '' : ' limit '.$limit;
		$sql = 'SELECT '.$field.' FROM `'.$this->db.'`.`'.$table.'`'. $where . $order . $limit;
		if(!$this->execute($sql)) return false;
		return $this->fetch_all($this->result);
	}

	/**
	 * 执行添加记录操作
	 * @param $data 		要增加的数据，参数为数组。数组key为字段值，数组值为数据取值
	 * @param $table 		数据表
	 * @param $return_id    要求返回的信息，默认(false)返回布尔值，表示成功与否，值为true时返回自增主键id值
	 * @param $replace		强制执行，存在相同结果时，主键的值将被改变
	 * @return integer
	 */
	public function insert(array $data, string $table = '', bool $return_id = false, bool $replace = false):Bool|Int{
		$table = $this->check_table($table);
		$field = implode(",", array_keys($data));
		$sqlval = array_values($data);
		$value = "";
		foreach($sqlval as $key => $val) $value .= "'".$val."',";
		$values = substr($value,0,-1);
		$cmd = $replace ? 'REPLACE' : 'INSERT';
		$sql = $cmd ." INTO `".$this->db.'`.`'.$table.'`('.$field.')'.' VALUES('.$values.')';
		if(!$this->execute($sql)) return false;
		return $return_id ? mysqli_insert_id($this->conn) : $this->result;
	}
	
	/**
	 * 执行更新记录操作
	 * @param $data 		要更新的数据内容，格式为数组或者字符串，建议数组。
	 * 						数组格式：key为字段值，value为数据值[例: array('name'=>'phpcms','password'=>'123456')]
	 * 						字符串格式[例：`name`='phpcms',`hits`=`hits`+1]。
	 * @param $where 		更新数据时的条件，格式为数组或字符串
	 * @param $aft_rows		是否返回受操作影响的行数（成功修改条数）
	 * @param $table 		数据表
	 * @return boolean or integer
	 */
	public function update(string|array $data, string|array $where='', bool $aft_rows=false, string $table=''):Bool|Int{
		$table = $this->check_table($table);
		$where = $this->check_where($where);
		$value = '';
		foreach($data as $key => $val) $value .= $key.'='."'$val',";
		$sqlval = substr($value,0,-1);
		$sql = "UPDATE `".$this->db.'`.`'.$table."` SET ".$sqlval.$where;
		if(!$this->execute($sql)) return false;
		return $aft_rows ? $this->conn->affected_rows : $this->result;
	}

	/**
	 * 执行删除记录操作
	 * @param $where 		删除数据条件,不充许为空。
	 * @param $table 		数据表，格式为数组或字符串
	 * @param $aft_rows		是否返回受操作影响的行数（成功删除条数）
	 * @return boolean
	 */
	public function delete(string|array $where='', bool $aft_rows=false, string $table=''):Bool|Int{
		$table = $this->check_table($table);
		$where = $this->check_where($where);
		$sql = 'DELETE FROM `'.$table.'`'.$where;
		if(!$this->execute($sql)) return false;
		return $aft_rows ? $this->conn->affected_rows : $this->result;
	}
		
	/**
	 * 检查是否设置表名
	 */
	public function check_table(string $table):String{
		if(empty($this->table_name) && empty($table)){
			exit("Table name cannot be empty");
		}else{
			$table = empty($table) ? $this->table_name : $table;
		}
		return $table;
	}

	/**
	 * 检查条件设置
	 */
	public function check_where(string|array $condition):String{
		if(empty($condition)){
			$where = "";
		}else{
			if(is_array($condition)){
				foreach($condition as $key => $val) $where = ' WHERE '.$key.'='."'$val'";
			}else{
				$where = ' WHERE '.$condition;
			}
		}
		if(empty($where)) die("Error：Please set the conditions");
		return $where;
	}

	/**
	 * 数据库SQL执行方法
	 * @param $sql 要执行的sql语句
	 * @return 查询资源句柄
	 */
	private function execute(string $sql):Bool{
		$this->result = $this->conn->query($sql);
		if(!$this->result) return false;
		return true;
	}

	/**
	 * 直接执行SQL语句
	 */
	public function query(string $sql):Bool|Object{
		$result = $this->execute($sql);
		return $result ? $this->result : false;
	}

	/**
	 * 释放结果集
	 */
	public function free_result(object $result):Void{
		mysqli_free_result($result);
	}
	
	/**
	 * 关闭MYSQL链接
	 */
	public function mysql_close(){
		$this->conn->close();
	}

	public function connect(){
		return $this->conn;
	}

}

/*********调用**********/
#$conn=new db_factroy();
#$conn->table_name="";

//查询
#$result=$conn->select();var_dump($result);
#$result=$conn->select("id<>1","id,create_time,user_nickname","id desc",'10');var_dump($result);

//添加
#$data=array('create_time'=>time(),'user_nickname'=>rand(10));
#$result=$conn->insert($data,'',true,true);var_dump($result);

//修改
#$result=$conn->update($data,array('id'=>52));var_dump($result);
#$result=$conn->update($data,"id=62");var_dump($result);

//删除
#$result=$conn->delete(['id'=>13]);
#$result=$conn->delete("id=58",1);

//使用SQL
#$result=$conn->fetch_all($conn->query("select * from ".$conn->table_name." where user_nickname like '%z%'"));var_dump($result);

?>
